// obj.h -- Dance objects definitions


// WORLD defines
#define MAXOBJECTS    1000
// Max number of forces in the world (max 32 supported)
#define MAXFORCES     32
// Max number of materials
#define MAXMATERIALS  50


// FORCES defines
#define F_GRAVITY     0x00000001


// ALLOC block granularity defines
#define ALL_OBJ       64
#define ALL_MASSES    128
#define ALL_SPRINGS   2048
#define ALL_CONNS     1024
#define ALL_POLYS     64



// STATUS defines
//   Common
#define S_NONE        0x00000000
#define S_ALIVE       0x00000001
#define S_WORLDFORCES 0x00000002
//   Mass only
#define S_TEMPFIXED   0x00000004
#define S_FIXED       0x00000008
#define S_BUMPGND     0x00000010
#define S_BUMPBC      0x00000020
#define S_NOCLRACC    0x00000040
#define S_STICKED     0x00000080
//   Objects only
#define S_AUTONORMS   0x00000100
#define S_AUTONORMV   0x00000200
#define S_DRAWSMOOTH  0x00000400
#define S_CULLBACK    0x00000800
#define S_CW          0x00001000
#define S_TEXTURED	  0x00002000
#define S_ENVCUBE     0x00004000


// MODE defines for function calls
#define M_GENSPRINGS  0x01000000
#define M_GENCONNS    0x02000000
#define M_GENPOLYS    0x04000000
#define M_3DSSWAPNORM 0x08000000


// MASS structure definition ---------------------------------------

typedef struct {
    /* Current position, velocity, acceleration */
    double x, y, z;
    double vx, vy, vz;
    double ax, ay, az;
    // Vertex normal
    double nx, ny, nz;

    // Polys count
    int npolys;

    /* Mass and radius of mass */
    double mass;
    double elastic;
    double radius;

    /* mass id */
    int mid;


    /* Connections to springs */
    //int *pars;
    //int num_pars;

    // Status
    int status;

    // Influence ID (bits are checked)
    DWORD flu;

} mass;


// SPRING structure definition ---------------------------------------

typedef struct {
    /* Ks, Kd and rest length of spring */
    double ks, kd;
    double restlen;

    /* Connected to masses m1 and m2 */
    int mid0;
    int mid1;
 
    /* spring id */
    int id;

    // Status
    int status;
} spring;




// FORCE structure definition ---------------------------------------
typedef struct {
    // fx, fy, fz : vector of force if not connected to mass
    double fx, fy, fz;

    // Strength coeff
    double strength;

    // Centered on mass mid0 OR -1 if absolute force
    int mid0;

    // Force (Influence) ID. ONE BIT PER FORCE: MAX 32 FORCES.
    long flu;

    // Status
    int status;
} force;


// POLY structure definition ---------------------------------------
// >>> Only triangles are supported in this release <<<
typedef struct {
    // Vertex are masses mid0, mid1, mid2
    int mid0;
    int mid1;
    int mid2;

    // Surface normal
    double nx, ny, nz;

    // Material id (0 to use default)
    int matid;

    // Id
    int id;

    // Status
    int status;
} poly;



// BOUNDING CUBE structure definition ---------------------------------------
typedef struct {
    // Centre pos
    double cx, cy, cz;

    // Axis rotations
    double rx, ry, rz;

    // Original vertecz position of 0-centered cube
    double vo[8][3];

    // Current vertecz position
    double vc[8][3];

	float	vm[36][2];
	int		txts[6];
    // Status
    int status;
} BCube;



// MATERIAL structure definition ---------------------------------------
typedef struct {
    // Name
    char name[128];

    // Id
    int id;

    // Ambient
    float amb_R, amb_G, amb_B, amb_A;
    // Diffuse
    float dif_R, dif_G, dif_B, dif_A;
    // Specular
    float spe_R, spe_G, spe_B, spe_A;
    // Emission
    float emi_R, emi_G, emi_B, emi_A;
    // Shininess
    float shi;

    // Status
    int status;
} material;


// OBJ structure definition ---------------------------------------
typedef struct {
  char    name[256];
  int     id;                 // Object numeric ID
  int     num_masses;         // Number of masses
  int     num_springs;        // Number of springs
  int     num_conns;          // Number of connections data (including tags)
  int     num_polys;          // Number of polygons
  int     num_subobj;         // Number of subobjects
  int     all_masses;         // Number of masses allocated
  int     all_springs;        // Number of springs allocated
  int     all_conns;          // Number of connections allocated
  int     all_polys;          // Number of polygons allocated
  int     all_subobj;         // Number of subobj_id allocated
  mass    *masses;            // Masses data
  spring  *springs;           // Springs data
  int     *conns;             // Connections data (smid, emid, emid, [...], -1). End all with -2.
  poly    *polys;             // Polygons data
  int     *subobj_id;         // Id of subobjects
  int     flags;              // Flags
  double  xmat[16];           // OGL xform matrix (relative to parent)
} Obj;


// WORLD structure definition ---------------------------------------
typedef struct {
  char     name[256];
  int      num_obj;                // Number of objects
  int      num_mat;                // Number of materials
  int      last_obj_id;            // Last object id assigned
  Obj      obj[MAXOBJECTS];        // Objects data
  int      forces_obj_id;          // Id of object containing forces
  long     busyforces;             // Mask containing all created forces
  force    forces[MAXFORCES];      // Forces data
  material mats[MAXMATERIALS];     // Materials data
  int      bcube_obj_id;           // BCube object id
  BCube    bcube;                  // Bounding cube
} World;

